/*
 * Copyright (C) 2003-2007 eXo Platform SAS.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see<http://www.gnu.org/licenses/>.
 */
package org.exoplatform.services.jcr.api.writing;

import java.io.ByteArrayInputStream;
import java.util.GregorianCalendar;

import javax.jcr.Node;
import javax.jcr.Property;
import javax.jcr.RepositoryException;
import javax.jcr.Value;

import org.exoplatform.services.jcr.JcrAPIBaseTest;
import org.exoplatform.services.jcr.impl.core.SessionImpl;

/**
 * Created by The eXo Platform SAS.
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov</a>
 * @version $Id: TestValue.java 35189 2009-08-07 14:35:29Z pnedonosko $
 */

public class TestValue extends JcrAPIBaseTest {

  public void initRepository() throws RepositoryException {
    Node root = session.getRootNode();
    Node propDef = root.addNode("propertyDefNode", "nt:unstructured");
    // Unknown Property Type. Should set something!
    propDef.setProperty("jcr:defaultValue", "testString");

    root.addNode("childNodeDefNode", "nt:unstructured");
    session.save();
  }

  public void tearDown() throws Exception {
    session = (SessionImpl) repository.login(credentials, WORKSPACE);
    Node root = session.getRootNode();
    root.getNode("propertyDefNode").remove();
    root.getNode("childNodeDefNode").remove();
    session.save();

    super.tearDown();
  }

  public void testSetValue() throws RepositoryException {
    Node root = session.getRootNode();
    Property property = root.getNode("propertyDefNode").getProperty("jcr:defaultValue");

    property.setValue("haha");
    property.setValue("default");
    property.setValue(new ByteArrayInputStream(new String("default").getBytes()));
    property.setValue(true);
    property.setValue(new GregorianCalendar());
    property.setValue(20D);
    property.setValue(20L);
    assertFalse("Property '" + property.getPath() + "' must be single-valued",
                property.getDefinition().isMultiple());
  }

  public void testSetMuliValue() throws RepositoryException {
    Node root = session.getRootNode();
    Node propertyDefNode = root.getNode("propertyDefNode");
    Property singleValuedProperty = propertyDefNode.getProperty("jcr:defaultValue");
    // remove initially created property
    singleValuedProperty.setValue((String) null);
    propertyDefNode.save();

    Property multiValuedProperty = propertyDefNode.setProperty("jcr:defaultValue", new String[] {
        null, null });

    Value[] values = { session.getValueFactory().createValue("not"),
        session.getValueFactory().createValue("in") };
    multiValuedProperty.setValue(values);

    assertTrue("Property '" + multiValuedProperty.getPath() + "' must be multi-valued",
               multiValuedProperty.getDefinition().isMultiple());
  }

}
